//----------------------------------------------------------------------
// File:			bd_fix_rad_search.cxx
// Programmer:		David Mount, Aleksandr Andrievskii
// Description:		Standard bd-tree search
// Last modified:	06/22/08
//----------------------------------------------------------------------
// Copyright 2008 Aleksandr Andrievskii
//
// Copyright (c) 1997-2005 University of Maryland and Sunil Arya and
// David Mount.  All Rights Reserved.
// 
// This file is part of the WANN (Weighted Approximate Nearest 
// Neighbors) library.
//
// WANN is free software: you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation, either version 3 of the
// License, or (at your option) any later version.
//
// WANN is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
// Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with WANN.  If not, see <http://www.gnu.org/licenses/>.
//----------------------------------------------------------------------
// Acknowledgment:
//
// The development of the WANN library has been supported by ANR
// (Agence Nationale de la Recherche, France) under contract 
// BLAN07-2_183172 OTARIE and by RFBR (Russian Foundation for Basic 
// Research) under contract RFBR/CNRS 07-01-92217.
//----------------------------------------------------------------------
// History:
//
// This file has been originally released under GNU LGPL as part of the ANN
// (Approximate Nearest Neighbors) library of David Mount and Sunil
// Arya.  It has been modified to become part of the WANN library.
//
// WANN history:
//  Revision 0.2 06/22/08
//      Modified ann_FR_search parameters.
//	Revision 0.1 06/03/08
//		Modified copying permision statement.
//
// ANN History:
//	Revision 1.1  05/03/05
//		Initial release
//----------------------------------------------------------------------

#include "bd_tree.h"					// bd-tree declarations
#include "kd_fix_rad_search.h"			// kd-tree FR search declarations

//----------------------------------------------------------------------
//	Approximate searching for bd-trees.
//		See the file kd_FR_search.cxx for general information on the
//		approximate nearest neighbor search algorithm.  Here we
//		include the extensions for shrinking nodes.
//----------------------------------------------------------------------

//----------------------------------------------------------------------
//	bd_shrink::ann_FR_search - search a shrinking node
//----------------------------------------------------------------------

void ANNbd_shrink::ann_FR_search(ANNdist box_dist
#ifdef WANN_TORUS
       ,ANNpoint			q
#endif
)
{
												// check dist calc term cond.
	if (ANNmaxPtsVisited != 0 && ANNptsVisited > ANNmaxPtsVisited) return;

	ANNdist inner_dist = 0;						// distance to inner box
	for (int i = 0; i < n_bnds; i++) {			// is query point in the box?
		if (bnds[i].out(ANNkdFRQ)) {			// outside this bounding side?
												// add to inner distance
			inner_dist = (ANNdist) ANN_SUM(inner_dist, bnds[i].dist(ANNkdFRQ));
		}
	}
	if (inner_dist <= box_dist) {				// if inner box is closer
		child[ANN_IN]->ann_FR_search(inner_dist);// search inner child first
		child[ANN_OUT]->ann_FR_search(box_dist);// ...then outer child
	}
	else {										// if outer box is closer
		child[ANN_OUT]->ann_FR_search(box_dist);// search outer child first
		child[ANN_IN]->ann_FR_search(inner_dist);// ...then inner child
	}
	ANN_FLOP(3*n_bnds)							// increment floating ops
	ANN_SHR(1)									// one more shrinking node
}
