//----------------------------------------------------------------------
// File:			kd_pr_search.h
// Programmer:		Sunil Arya and David Mount,
//					Andrei Sobolevskii and Aleksandr Andrievskii
// Description:		Priority kd-tree search
// Last modified:	06/22/08
//----------------------------------------------------------------------
// Copyright 2008 Andrei Sobolevskii and Aleksandr Andrievskii
//
// Copyright (c) 1997-2005 University of Maryland and Sunil Arya and
// David Mount.  All Rights Reserved.
// 
// This file is part of the WANN (Weighted Approximate Nearest 
// Neighbors) library.
//
// WANN is free software: you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation, either version 3 of the
// License, or (at your option) any later version.
//
// WANN is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
// Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with WANN.  If not, see <http://www.gnu.org/licenses/>.
//----------------------------------------------------------------------
// Acknowledgment:
//
// The development of the WANN library has been supported by ANR
// (Agence Nationale de la Recherche, France) under contract 
// BLAN07-2_183172 OTARIE and by RFBR (Russian Foundation for Basic 
// Research) under contract RFBR/CNRS 07-01-92217.
//----------------------------------------------------------------------
// History:
//
// This file has been originally released under GNU LGPL as part of the ANN
// (Approximate Nearest Neighbors) library of David Mount and Sunil
// Arya.  It has been modified to become part of the WANN library.
//
// WANN history:
//	Revision 0.2 06/22/08
//		New priority queue ANNprPointPQ
//	Revision 0.1 06/03/08
//		Modified copyright information.
//		Changes comments.
//		Added global array ANNprWeights.
//		
// ANN history:
//	Revision 0.1  03/04/98
//		Initial release
//----------------------------------------------------------------------

#ifndef ANN_kd_pr_search_H
#define ANN_kd_pr_search_H

#include "kd_tree.h"					// kd-tree declarations
#include "kd_util.h"					// kd-tree utilities
#include "pr_queue.h"					// priority queue declarations
#include "pr_queue_k.h"					// k-element priority queue

#include <wann/ANNperf.h>				// performance evaluation

//----------------------------------------------------------------------
//	Global variables
//		Active for the life of each call to Appx_Near_Neigh() or
//		Appx_k_Near_Neigh().
//----------------------------------------------------------------------

extern double			ANNprEps;		// the error bound
extern int				ANNprDim;		// dimension of space
extern ANNpoint			ANNprQ;			// query point
extern double			ANNprMaxErr;	// max tolerable squared error
extern ANNpointArray	ANNprPts;		// the points
#ifndef ANN_NO_WEIGHTS
extern ANNdistArray		ANNprWeights;	// the weights
#endif
extern ANNpr_queue		*ANNprBoxPQ;	// priority queue for boxes
extern ANNmin_k			*ANNprPointMK;	// set of k closest points
#ifdef WANN_TORUS
// when we search in toroidal space we must pass a query point to
// the node, and some of the coordinates may be inverted
extern ANNpr_queue		*ANNprPointPQ;	// priority queue for points
#endif

#endif
